using Microsoft.Extensions.Logging.Abstractions;
using Piwigo.Client.Session;

namespace Piwigo.Client.Tests;

[TestFixture]
public class SessionApiTests : ApiTestsBase
{
    private SessionApi _sessionApi = null!;

    protected override void OnSetUp()
    {
        base.OnSetUp();
        _sessionApi = new SessionApi(Context, new NullLogger<SessionApi>());
    }

    [Test]
    public async Task Login_should_set_cookies_and_session()
    {
        await LoginAsync();
        CorrectMethodShouldGetCalled("pwg.session.login");
    }

    [Test]
    public async Task GetStatus_should_return_config()
    {
        await LoginAsync();
        SetJsonResult(GetStatusResponse());

        var status = await _sessionApi.GetStatusAsync();

        status.Should().NotBeNull();
        status.Username.Should().Be("admin");
        CorrectMethodShouldGetCalled("pwg.session.getStatus");
    }

    [Test]
    public async Task GetUploadChunkSizeAsync_should_get_size_from_session()
    {
        await LoginAsync();
        SetJsonResult(GetStatusResponse());

        var chunkSize = await _sessionApi.GetUploadChunkSizeAsync();

        chunkSize.Should().Be(512);
        CorrectMethodShouldGetCalled("pwg.session.getStatus");
    }

    [Test]
    public async Task GetSupportedFileTypes_should_get_allowed_types_from_session()
    {
        await LoginAsync();
        SetJsonResult(GetStatusResponse());

        var fileTypes = await _sessionApi.GetSupportedFileTypes();

        fileTypes.Should().HaveCount(2);
        fileTypes.Should().Contain("jpg");
        fileTypes.Should().Contain("png");
        CorrectMethodShouldGetCalled("pwg.session.getStatus");
    }

    [Test]
    public async Task GetUploadChunkSizeAsync_should_only_get_size_from_session_once()
    {
        await LoginAsync();
        SetJsonResult(GetStatusResponse());

        var chunkSize = await _sessionApi.GetUploadChunkSizeAsync();
        chunkSize.Should().Be(512);
        CorrectMethodShouldGetCalled("pwg.session.getStatus");

        ResetHttpTest();
        chunkSize = await _sessionApi.GetUploadChunkSizeAsync();
        chunkSize.Should().Be(512);
        HttpTest?.CallLog.Should().BeEmpty();
    }

    [Test]
    public async Task Logout_should_set_IsLoggedIn_to_false()
    {
        await LoginAsync();

        SetOkResult();

        await _sessionApi.LogoutAsync();
        Context.IsLoggedIn.Should().BeFalse();
        CorrectMethodShouldGetCalled("pwg.session.logout");
    }

    private async Task LoginAsync()
    {
        HttpTest?.RespondWith("{}", 200, cookies: new { pwg_id = "pwg_id" });
        var sessionApi = new SessionApi(Context, new NullLogger<SessionApi>());
        await sessionApi.LoginAsync();
        Context.IsLoggedIn.Should().BeTrue();
    }

    private static PiwigoResponse<SessionStatus> GetStatusResponse()
    {
        return new PiwigoResponse<SessionStatus>
        {
            Status = "OK",
            Result = new SessionStatus
            {
                Username = "admin",
                Version = "12.0.0",
                UploadFormChunkSize = 512,
                UploadFileTypes = "jpg,png"
            }
        };
    }
}