using Microsoft.Extensions.Logging;

namespace Piwigo.Client.Session;

public class SessionApi : ISessionApi
{
    private readonly IPiwigoContext _context;
    private readonly ILogger<SessionApi> _logger;
    private SessionStatus? _currentStatus;

    public SessionApi(IPiwigoContext context, ILogger<SessionApi> logger)
    {
        _context = context ?? throw new ArgumentNullException(nameof(context));
        _logger = logger ?? throw new ArgumentNullException(nameof(logger));
    }

    public async Task LogoutAsync(CancellationToken cancellationToken = default)
    {
        await _context.LogoutAsync(cancellationToken);
        _currentStatus = null;
    }

    public Task LoginAsync(CancellationToken cancellationToken = default)
    {
        return _context.LoginAsync(cancellationToken);
    }

    public async Task<SessionStatus> GetStatusAsync(CancellationToken cancellationToken = default)
    {
        var typedResponse = await _context.PostAsync<PiwigoResponse<SessionStatus>>(_logger, "pwg.session.getStatus", cancellationToken);
        return typedResponse.Result;
    }

    public async ValueTask<int> GetUploadChunkSizeAsync(CancellationToken cancellationToken = default)
    {
        const int minChunkSize = 512;
        var status = await GetCachedSessionStatusAsync(cancellationToken);
        return Math.Max(status.UploadFormChunkSize, minChunkSize);
    }

    public async ValueTask<IReadOnlySet<string>> GetSupportedFileTypes(CancellationToken cancellationToken = default)
    {
        var status = await GetCachedSessionStatusAsync(cancellationToken);
        var supportedFileTypes = status.UploadFileTypes?.Split(",").Select(s => s.ToLower()).ToHashSet(StringComparer.OrdinalIgnoreCase);
        if (supportedFileTypes is null)
        {
            throw new PiwigoException("Piwigo server seems to be misconfigured as the session does not provide any filetype to upload");
        }

        return supportedFileTypes;
    }

    private async ValueTask<SessionStatus> GetCachedSessionStatusAsync(CancellationToken cancellationToken)
    {
        if (_currentStatus is not null)
        {
            return _currentStatus;
        }

        _currentStatus = await GetStatusAsync(cancellationToken);
        return _currentStatus;
    }
}