namespace Piwigo.Client.Tags;

public interface ITagApi
{
    /// <summary>
    ///     Adds a new tag to the gallery
    /// </summary>
    /// <param name="name">the name of the tag</param>
    /// <param name="cancellationToken">
    ///     <see cref="CancellationToken" />
    /// </param>
    /// <returns>Information about the uploaded tag</returns>
    Task<AddedTag> AddAsync(string name, CancellationToken cancellationToken = default);

    /// <summary>
    ///     Removes assignments and deletes the tag form the gallery
    /// </summary>
    /// <param name="tagId">the id of the tag</param>
    /// <param name="apiToken">The API token that can be read from <see cref="Piwigo.Client.Session.SessionStatus" /></param>
    /// <param name="cancellationToken">
    ///     <see cref="CancellationToken" />
    /// </param>
    /// <returns></returns>
    Task DeleteAsync(int tagId, string apiToken, CancellationToken cancellationToken = default);

    /// <summary>
    ///     Duplicates an existing tag and returns the information of the newly added tag
    /// </summary>
    /// <param name="tagId">tag id to duplicate</param>
    /// <param name="targetName">the name of the new tag</param>
    /// <param name="apiToken">The API token that can be read from <see cref="Piwigo.Client.Session.SessionStatus" /></param>
    /// <param name="cancellationToken">
    ///     <see cref="CancellationToken" />
    /// </param>
    /// <returns>The information about the added tag</returns>
    Task<AddedTag> DuplicateAsync(int tagId, string targetName, string apiToken, CancellationToken cancellationToken = default);

    /// <summary>
    ///     Gets a list of all known tags. This call is only available for admins
    /// </summary>
    /// <param name="cancellationToken">
    ///     <see cref="CancellationToken" />
    /// </param>
    /// <returns>all available tags</returns>
    Task<IReadOnlyCollection<Tag>> GetAdminListAsync(CancellationToken cancellationToken = default);

    /// <summary>
    ///     Gits a list of known tags that are at least assigned to one image
    /// </summary>
    /// <param name="sortByCounter">sorts the result by the number of assigned images of a tag</param>
    /// <param name="cancellationToken">
    ///     <see cref="CancellationToken" />
    /// </param>
    /// <returns>all tags with at least one image assigned</returns>
    Task<IReadOnlyCollection<AssignedTag>> GetListAsync(bool? sortByCounter = null, CancellationToken cancellationToken = default);

    /// <summary>
    ///     Merges two existing tags
    /// </summary>
    /// <param name="targetTagId">The tag where the images should get merged into</param>
    /// <param name="sourceTagId">The tag that will get obsolete</param>
    /// <param name="apiToken">The API token that can be read from <see cref="Piwigo.Client.Session.SessionStatus" /></param>
    /// <param name="cancellationToken">
    ///     <see cref="CancellationToken" />
    /// </param>
    /// <returns>Information about the merge result</returns>
    Task<MergeResult> MergeAsync(int targetTagId, int sourceTagId, string apiToken, CancellationToken cancellationToken = default);

    /// <summary>
    ///     renames a existing tag
    /// </summary>
    /// <param name="tagId">the tag id that gets renamed</param>
    /// <param name="newName">the new name of the existing tag</param>
    /// <param name="apiToken">The API token that can be read from <see cref="Piwigo.Client.Session.SessionStatus" /></param>
    /// <param name="cancellationToken">
    ///     <see cref="CancellationToken" />
    /// </param>
    /// <returns></returns>
    Task RenameAsync(int tagId, string newName, string apiToken, CancellationToken cancellationToken = default);
}