using Microsoft.Extensions.Logging;

namespace Piwigo.Client.Tags;

public class TagApi : ITagApi
{
    private readonly IPiwigoContext _context;
    private readonly ILogger<TagApi> _logger;

    public TagApi(IPiwigoContext context, ILogger<TagApi> logger)
    {
        _context = context ?? throw new ArgumentNullException(nameof(context));
        _logger = logger ?? throw new ArgumentNullException(nameof(logger));
    }

    public async Task<AddedTag> AddAsync(string name, CancellationToken cancellationToken = default)
    {
        var formParams = new Dictionary<string, string>
        {
            { "name", name }
        };
        var response = await _context.PostAsync<PiwigoResponse<AddedTag>>(_logger, "pwg.tags.add", formParams, cancellationToken);
        return response.Result;
    }

    public async Task<IReadOnlyCollection<Tag>> GetAdminListAsync(CancellationToken cancellationToken = default)
    {
        var response = await _context.PostAsync<PiwigoResponse<TagList<Tag>>>(_logger, "pwg.tags.getAdminList", new Dictionary<string, string>(), cancellationToken);
        return response.Result.Tags;
    }

    public async Task<IReadOnlyCollection<AssignedTag>> GetListAsync(bool? sortByCounter = null, CancellationToken cancellationToken = default)
    {
        var formParams = new Dictionary<string, string>();
        formParams.AddIfValueNotNull("sort_by_counter", sortByCounter?.ToString());

        var response = await _context.PostAsync<PiwigoResponse<TagList<AssignedTag>>>(_logger, "pwg.tags.getList", formParams, cancellationToken);
        return response.Result.Tags;
    }

    public async Task DeleteAsync(int tagId, string apiToken, CancellationToken cancellationToken = default)
    {
        var formParams = new Dictionary<string, string>
        {
            { "tag_id", tagId.ToString() },
            { "pwg_token", apiToken }
        };

        await _context.PostAsync<PiwigoResponse>(_logger, "pwg.tags.delete", formParams, cancellationToken);
    }

    public async Task<AddedTag> DuplicateAsync(int tagId, string targetName, string apiToken, CancellationToken cancellationToken = default)
    {
        var formParams = new Dictionary<string, string>
        {
            { "tag_id", tagId.ToString() },
            { "copy_name", targetName },
            { "pwg_token", apiToken }
        };

        var response = await _context.PostAsync<PiwigoResponse<AddedTag>>(_logger, "pwg.tags.duplicate", formParams, cancellationToken);

        return response.Result;
    }

    public async Task<MergeResult> MergeAsync(int targetTagId, int sourceTagId, string apiToken, CancellationToken cancellationToken = default)
    {
        var formParams = new Dictionary<string, string>
        {
            { "destination_tag_id", targetTagId.ToString() },
            { "merge_tag_id", sourceTagId.ToString() },
            { "pwg_token", apiToken }
        };

        var response = await _context.PostAsync<PiwigoResponse<MergeResult>>(_logger, "pwg.tags.merge", formParams, cancellationToken);

        return response.Result;
    }

    public async Task RenameAsync(int tagId, string newName, string apiToken, CancellationToken cancellationToken = default)
    {
        var formParams = new Dictionary<string, string>
        {
            { "tag_id", tagId.ToString() },
            { "new_name", newName },
            { "pwg_token", apiToken }
        };

        await _context.PostAsync<PiwigoResponse>(_logger, "pwg.tags.rename", formParams, cancellationToken);
    }
}